# Components enabled if supported by target architecture:
%ifarch %ix86 x86_64
  %bcond_without gold
%else
  %bcond_with gold
%endif

%global git_hash 7913836381e3f3984b711f921a1cf9da5d37e107
%global ver_major_minor 5.0
%global exec_suffix -%{ver_major_minor}
%global install_prefix %{_libdir}/%{name}
%global llvm_bindir %{_libdir}/llvm-%{ver_major_minor}/bin
%global llvm_includedir %{_includedir}/llvm-%{ver_major_minor}
%global llvm_libdir %{_libdir}

Name:		llvm%{ver_major_minor}
Version:	%ver_major_minor.0
Release:	3%{?dist}
Summary:	The Low Level Virtual Machine

License:	NCSA
URL:		http://llvm.org
#Source0:	http://llvm.org/releases/%{version}/llvm-%{version}.src.tar.xz
Source0: llvm-%{git_hash}.zip

# recognize s390 as SystemZ when configuring build
Patch0:		llvm-3.7.1-cmake-s390.patch
Patch2: 	0001-Fix-llvm-config-paths-on-Fedora.patch

BuildRequires:	cmake
BuildRequires:	zlib-devel
BuildRequires:  libffi-devel
BuildRequires:	ncurses-devel
BuildRequires:	python3-sphinx
BuildRequires:	multilib-rpm-config
%if %{with gold}
BuildRequires:  binutils-devel
%endif
BuildRequires:  libstdc++-static

Requires:	%{name}-libs%{?_isa} = %{version}-%{release}

%description
LLVM is a compiler infrastructure designed for compile-time, link-time,
runtime, and idle-time optimization of programs from arbitrary programming
languages. The compiler infrastructure includes mirror sets of programming
tools as well as libraries with equivalent functionality.

%package devel
Summary:	Libraries and header files for LLVM
Requires:	%{name}%{?_isa} = %{version}-%{release}
Requires(post): %{_sbindir}/alternatives
Requires(postun): %{_sbindir}/alternatives

%description devel
This package contains library and header files needed to develop new native
programs that use the LLVM infrastructure.

%package doc
Summary:	Documentation for LLVM
BuildArch:	noarch
Requires:	%{name} = %{version}-%{release}

%description doc
Documentation for the LLVM compiler infrastructure.

%package libs
Summary:	LLVM shared libraries
Obsoletes: llvm-libs < %{version}-%{release}

%description libs
Shared libraries for the LLVM compiler infrastructure.

%package -n llvm-devel
Summary: Meta package for llvm development files.
Requires: llvm = %{version}-%{release}
Requires: %{name}-devel = %{version}-%{release}
Obsoletes: llvm-devel < %{version}-%{release}

%description -n llvm-devel
Meta package for llvm development files.

%package -n llvm-libs
Summary: Meta package for llvm shared libraries.
Requires: %{name}-libs = %{version}-%{release}

%description -n llvm-libs
Meta package fro llvm shared libraries.

%package -n llvm-static
Summary:	LLVM static libraries.
Obsoletes: llvm-static < %{version}-%{release}

%description -n llvm-static
Static libraries for the LLVM compiler infrastructure.

%package -n llvm
Summary: Meta package for llvm
Requires: %{name}%{?_isa} = %{version}-%{release}

%description -n llvm
Meta package for llvm

%prep
#autosetup -n llvm-%{version}.src -p1
%autosetup -n llvm-%{git_hash} -p1

%ifarch armv7hl

# These tests are marked as XFAIL, but they still run and hang on ARM.
for f in `grep -Rl 'XFAIL.\+arm' test/ExecutionEngine `; do  rm $f; done

%endif

%build
mkdir -p _build
cd _build

%ifarch s390
# Decrease debuginfo verbosity to reduce memory consumption during final library linking
%global optflags %(echo %{optflags} | sed 's/-g /-g1 /')
%endif

# force off shared libs as cmake macros turns it on.
%cmake .. \
	-DBUILD_SHARED_LIBS:BOOL=OFF \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
	-DCMAKE_SHARED_LINKER_FLAGS="-Wl,-Bsymbolic -static-libstdc++" \
%ifarch s390
	-DCMAKE_C_FLAGS_RELWITHDEBINFO="%{optflags} -DNDEBUG" \
	-DCMAKE_CXX_FLAGS_RELWITHDEBINFO="%{optflags} -DNDEBUG" \
%endif
%if 0%{?__isa_bits} == 64
	-DLLVM_LIBDIR_SUFFIX=64 \
%else
	-DLLVM_LIBDIR_SUFFIX= \
%endif
	\
	-DLLVM_TARGETS_TO_BUILD="X86;AMDGPU;PowerPC;NVPTX;SystemZ;AArch64;ARM;Mips;BPF" \
	-DLLVM_ENABLE_LIBCXX:BOOL=OFF \
	-DLLVM_ENABLE_ZLIB:BOOL=ON \
	-DLLVM_ENABLE_FFI:BOOL=ON \
	-DLLVM_ENABLE_RTTI:BOOL=ON \
	-DLLVM_TOOLS_INSTALL_DIR=%{_libdir}/llvm-%{ver_major_minor}/bin \
%if %{with gold}
	-DLLVM_BINUTILS_INCDIR=%{_includedir} \
%endif
	\
	-DLLVM_BUILD_RUNTIME:BOOL=ON \
	\
	-DLLVM_INCLUDE_TOOLS:BOOL=ON \
	-DLLVM_BUILD_TOOLS:BOOL=ON \
	\
	-DLLVM_INCLUDE_TESTS:BOOL=ON \
	-DLLVM_BUILD_TESTS:BOOL=ON \
	\
	-DLLVM_INCLUDE_EXAMPLES:BOOL=ON \
	-DLLVM_BUILD_EXAMPLES:BOOL=OFF \
	\
	-DLLVM_INCLUDE_UTILS:BOOL=ON \
	-DLLVM_INSTALL_UTILS:BOOL=OFF \
	\
	-DLLVM_INCLUDE_DOCS:BOOL=ON \
	-DLLVM_BUILD_DOCS:BOOL=ON \
	-DLLVM_ENABLE_SPHINX:BOOL=ON \
	-DLLVM_ENABLE_DOXYGEN:BOOL=OFF \
	\
	-DLLVM_BUILD_LLVM_DYLIB:BOOL=ON \
	-DLLVM_DYLIB_EXPORT_ALL:BOOL=ON \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
	-DLLVM_BUILD_EXTERNAL_COMPILER_RT:BOOL=ON \
	-DLLVM_INSTALL_TOOLCHAIN_ONLY:BOOL=OFF \
	\
	-DSPHINX_WARNINGS_AS_ERRORS=OFF \
	-DLLVM_INSTALL_SPHINX_HTML_DIR=%{_pkgdocdir}/html \
	-DSPHINX_EXECUTABLE=%{_bindir}/sphinx-build-3

make %{?_smp_mflags}

%install
cd _build
make install DESTDIR=%{buildroot}

# Add version suffix to binaries
mkdir -p %{buildroot}/%{_bindir}
for f in `ls %{buildroot}/%{_libdir}/llvm-%{ver_major_minor}/bin/*`; do
  filename=`basename $f`
  ln -s %{_libdir}/llvm-%{ver_major_minor}/bin/$filename %{buildroot}/%{_bindir}/$filename%{exec_suffix}
  ln -s %{_libdir}/llvm-%{ver_major_minor}/bin/$filename %{buildroot}/%{_bindir}/$filename
done

rm %{buildroot}%{_bindir}/llvm-config
ln -s %{_libdir}/llvm-%{ver_major_minor}/bin/llvm-config %{buildroot}/%{_bindir}/llvm-config%{exec_suffix}-%{__isa_bits}

# Move header files
mkdir -p %{buildroot}/%{_includedir}/llvm-%{ver_major_minor}/
mv %{buildroot}/%{_includedir}/llvm %{buildroot}/%{_includedir}/llvm-%{ver_major_minor}/
mv %{buildroot}/%{_includedir}/llvm-c %{buildroot}/%{_includedir}/llvm-%{ver_major_minor}/

# Move cmake files
mv %{buildroot}/%{_libdir}/cmake/{llvm,llvm-%{ver_major_minor}}

# Add symlink to headers and cmake files
ln -s %{_includedir}/llvm-%{ver_major_minor}/llvm %{buildroot}/%{_includedir}/llvm
ln -s %{_includedir}/llvm-%{ver_major_minor}/llvm-c %{buildroot}/%{_includedir}/llvm-c
ln -s %{_libdir}/cmake/llvm-%{ver_major_minor} %{buildroot}/%{_libdir}/cmake/llvm

# Move man pages to correct prefix
#mkdir -p %{buildroot}/%{_mandir}
#mv %{buildroot}/%{_mandir}/* %{buildroot}/%{_mandir}

%check
cd _build
#make check-all || :

export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:%{buildroot}/%{_libdir}
test "`%{buildroot}/%{llvm_bindir}/llvm-config --bindir`" -ef "%{buildroot}/%{llvm_bindir}"
test "`%{buildroot}/%{llvm_bindir}/llvm-config --libdir`" -ef "%{buildroot}/%{llvm_libdir}"
test "`%{buildroot}/%{llvm_bindir}/llvm-config --includedir`" -ef "%{buildroot}/%{llvm_includedir}"
test "`%{buildroot}/%{llvm_bindir}/llvm-config --cmakedir`" -ef "%{buildroot}/%{_libdir}/cmake/llvm-%{ver_major_minor}"

%post libs -p /sbin/ldconfig
%postun libs -p /sbin/ldconfig

%post -n llvm-devel
%{_sbindir}/update-alternatives --install %{_bindir}/llvm-config llvm-config %{_bindir}/llvm-config%{exec_suffix}-%{__isa_bits} %{__isa_bits}

%postun -n llvm-devel
if [ $1 -eq 0 ]; then
  %{_sbindir}/update-alternatives --remove llvm-config %{_bindir}/llvm-config%{exec_suffix}-%{__isa_bits}
fi

%files
%{_bindir}/*-%{ver_major_minor}
%exclude %{_bindir}/*
%{_libdir}/llvm-%{ver_major_minor}/bin/*
%exclude %{_bindir}/llvm-config%{exec_suffix}-%{__isa_bits}
%exclude %{_mandir}/man1/llvm-config.1.gz

%files -n llvm
%exclude %{_bindir}/*-%{ver_major_minor}
%exclude %{_bindir}/llvm-config%{exec_suffix}-%{__isa_bits}
%{_bindir}/*
%{_mandir}/man1/*.1.gz

%files libs
%{_libdir}/libLLVM-%{ver_major_minor}*.so
%{_libdir}/libLTO.so*
%exclude %{_libdir}/libLTO.so

%files -n llvm-libs
%{_libdir}/BugpointPasses.so
%{_libdir}/LLVMHello.so
%if %{with gold}
%{_libdir}/LLVMgold.so
%endif

%files devel
%{_bindir}/llvm-config%{exec_suffix}-%{__isa_bits}
%{_includedir}/llvm-%{ver_major_minor}/llvm
%{_includedir}/llvm-%{ver_major_minor}/llvm-c
%{_libdir}/cmake/llvm-%{ver_major_minor}

%files -n llvm-devel
%{_includedir}/llvm
%{_includedir}/llvm-c
%{_libdir}/cmake/llvm
%{_libdir}/libLLVM.so
%{_libdir}/libLTO.so
%{_mandir}/man1/llvm-config.1.gz

%files doc
%doc %{_pkgdocdir}/html

%files -n llvm-static
%{_libdir}/*.a

%changelog
* Fri Apr 21 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-1
- Initial version.
